% Main function that performs moment matching
%
% Input:
% ------
% moms:      moments to match, starting with the first moment. The number
%            of moments must be odd.
% sumOrders: sum of the multiplicities of the eigenvalues. It is in a
%            direct relation with the size of the resulting PH. It can also
%            be a vector.
%
% Output:
% -------
% ph: array of struct, with the solution ph
function ph = mommatch(moms, sumOrders)

    if ~exist('sumOrders','var')
        sumOrders = length(moms);
    end

    if rem(length(moms),2)==0
        error('Number of moments must be odd!');
    end
    
    if min(sumOrders)<length(moms)/2
        error('SumOrders parameter must be greater or equal than length(moms)/2!');
    end

    % check if phcsolve is configured
    global phcloc;
    if isempty(phcloc)
        error('Please set up PHClab first! Add PHClab directory to MATLAB path (see ''addpath'' function), and call ''set_phcpah'' with the path to the PHC binary.');
    end
    
    % normalize moments for better numerical behavior
    mean = moms(1);
    moms = reshape(moms,length(moms),1) ./ (mean.^(1:length(moms)))';
    
    % convert moments to reduced moments
    rmoms = moms ./ factorial(1:length(moms))';
    rmoms = [1.0; rmoms];
    
    % try matching with all entries of the 'sumOrders' input parameter
    allsol = [];
    for i=sumOrders
        fprintf('Trying to fit with R=%d... ', i);       
        
        % all possible configurations of the orders of the eigenvalues
        ord = allorders (length(rmoms)/2, i);
        sols = [];
        for k=1:size(ord,1)
            sols = [sols, matchwithorders(rmoms,ord(k,:))];
        end
        
        fprintf('%d solutions found.\n', length(sols));
        % add the newly found solutions to the vector to be returned
        allsol = [allsol, sols];
    end
    
    % before returning, adjust mean value which has been lost due
    % to the normalization
    for i=1:length(allsol)
        ph(i).alpha = allsol(i).alpha;
        ph(i).A = allsol(i).A /mean;
    end
end