% Solves the polynomial system of equations for moment matching
%
% Input:
% ------
% rmoms: vector of reduced moments (starting with moment 0)
% orders: the order of eigenvalues
%
% Output:
% -------
% a matrix, where each row corresponds to a solution
% each solution consists of weights and eigenvalues
%
function results = solvepolysys (rmoms, orders)

N = length(orders);
M = length(rmoms);

% pre-calculate binomial coefficients
bcoeff = zeros(M,N);
for j=1:N
    bcoeff(1,j) = 1.0;
    for n=1:M-1        
        bcoeff(n+1,j) = bcoeff(n,j) * (orders(j)-1+n) / n;
    end
%     for n=0:M-1
%         bcoeff(n+1,j) = factorial(orders(j)-1+n) / factorial(n) / factorial(orders(j)-1);
%     end
end

% construct polynomial system
eq = [];
for k=0:M-1
    for j=1:N
        term = zeros(1,2*N+1);
        term(1) = bcoeff(k+1,j);
        term(1+j) = 1;
        term(1+N+j) = k;
        eq = [eq; term];
    end
    eq = [eq; [-rmoms(k+1),zeros(1,2*N)]];
    eq = [eq; zeros(1,2*N+1)];    
end

%set_phcpath('C:\Users\gabor\Work\momatch\PHC\phc.exe');
%set_phcpath('/media/webspn/tex/newpubs/mommatch/code/PHC/phc');

% call phcsolve
make_system(eq);
try
    s = solve_system(eq);
catch
    results = [];
    return;
end
%s = refine_sols(eq,s,1.0e-15,1.0e-15,1.0e-15,200);
%format long g;

% collect solutions
msol = [];
for l=1:length(s)
    sol = [];
    for j=1:2*N
        sol = [sol, s(l).(strcat('x','0'+j))];
    end
    ix = abs(imag(sol))<1e-14;
    sol(ix) = real(sol(ix));
    if min(real(sol(N+1:2*N)))>0 && sum(isinf(sol))==0
        lamb = 1./sol(N+1:2*N);
        % sort solution to allow selecting the unique ones later
        % skip those that are invalid (complex dominiant eigenvalues or
        % eigenvalues with positive real part)
        ix=(1:N);
        for n=1:N
            for m=1:N-n
                if (isreal(lamb(m)) && isreal(lamb(m+1)) && lamb(m+1)<lamb(m)) ...
                    || (~isreal(lamb(m)) && isreal(lamb(m+1))) ...
                    || (~isreal(lamb(m)) && ~isreal(lamb(m+1)) && imag(lamb(m))>0 && imag(lamb(m+1))<0) ...
                    || (~isreal(lamb(m)) && ~isreal(lamb(m+1)) && sign(imag(lamb(m)))==sign(imag(lamb(m+1))) && abs(lamb(m+1))<abs(lamb(m)))
                    tmp = lamb(m);
                    lamb(m) = lamb(m+1);
                    lamb(m+1) = tmp;
                    tmp = ix(m);
                    ix(m) = ix(m+1);
                    ix(m+1) = tmp;
                end
            end
        end
        if isreal(lamb(1))
            msol = [msol; [sol(ix),lamb,orders(ix),ix]];
        end
    end
end

% keep only unique solutions
solutions = [];
for i=1:size(msol,1)
    found = 0;
    for j=1:size(solutions,1)
        if max(abs(msol(i,1:3*N) - solutions(j,1:3*N)))<1e-9
            found = 1;
            break;
        end
    end
    if found==0
        solutions = [solutions; msol(i,:)];
    end
end

%total = size(solutions,1)

% return with the proper state ordering
results = zeros(size(solutions,1), 2*N);
for i=1:size(solutions,1)
    ralpha = solutions(i,1:N);
    rlambda = solutions(i,N+1:2*N);
    ix = solutions(i,3*N+1:4*N);
    alpha = zeros(1,N);
    lambda = zeros(1,N);
    alpha(ix) = ralpha;
    lambda(ix) = rlambda;
    results(i,:) = [alpha, lambda];
end

